
/* 
   Basic code for the UV_2 Click. ac:pic
   VEML6075 https://www.vishay.com/docs/84304/veml6075.pdf
   For mikroC Pro for PIC32
   EasyPIC Fusion v7.
   PIC32MX795F512L.
   Just need to add some pointers.
   By Mark Joyce.

   24.12.17
*/
// Set SW11 #1 to ON.(EasyTFT BCK LIGHT)
// Every other switch is OFF! (Not GND on tri states)
// UV 2 Click in mikroBUS port 1 or 2.

#include "Resources.h" // Just some TFT stuff.

//VEML6075 Commands and constants
#define VEML6075_UV_CONF         0x00
#define VEML6075_UVA_DATA        0x07
#define VEML6075_UV_DUMMY_DATA   0x08
#define VEML6075_UVB_DATA        0x09
#define VEML6075_UVCOMP1_DATA    0x0A
#define VEML6075_UVCOMP2_DATA    0x0B
#define VEML6075_UV_ID           0x0C     // returns 0x26 or 38(DEC)
//Set for OPEN AIR. Page 15.
//http://www.vishay.com/docs/84339/designingveml6075.pdf
#define A                        2.22
#define B                        1.33
#define C                        2.95
#define D                        1.74
#define UVAresponse              0.001461 // Or you can try these see page 14 ->,   0.0011
#define UVBresponse              0.002591                                         //0.00125

#define VEML6075_WADD             0x20 // Write Address
#define VEML6075_RADD             0x21 // Read Address

 //Integration Time
const enum IT
{
 IT_50 = 0, //50ms
 IT_100,    //100ms etc
 IT_200,
 IT_400,
 IT_800
 };
 
 //light sensitivity increases with integration time.
 const short IT = IT_100;  // Integration time. 

 //Function Prototypes
 void InitVEML6075(void);
 unsigned int get16(unsigned int);
 float getUVAIntensity(int);
 float getUVBIntensity(int);
 float getUVIndex(unsigned int n[]); // TODO Use pointers.
 
void main()
{
     unsigned short i;          // Counter
     char display[20];          // Some String.
     unsigned int UVData[5];    // {UVA, UVDummy, UVB, UVComp1, UVComp2}
     float UVi;                 // UV Index

     AD1PCFG = 0xFFFF;          // Configure AN pins as digital I/O
     JTAGEN_bit = 0;            // Disable JTAG

     // Initialize the I2C2 module with clock_rate of 100000
     I2C2_Init(100000);
     Delay_ms(100);

     // Initialize the TFT for some feedback.
     TFT_BLED_Direction = 0;                 // Set TFT backlight pin as output
     TFT_Init_ILI9341_8bit(320, 240);        // Initialize TFT display
     TFT_BLED = 1;                           // Turn on TFT backlight
     
     //Greeting Screen 1.
     TFT_Fill_Screen(CL_BLACK);
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
     TFT_Write_Text("UV 2 Click Example", 70, 10);
     TFT_Write_Text("By Mark Joyce", 10, 210);


     //Is the VEML6075 on the I2C(#2) Bus or somethng else?
     if (get16(VEML6075_UV_ID) != 38)
     {
        TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
        TFT_Write_Text("Device ERROR VEML6075 ",10,50);
        while(1); //Lock it up.
     }
     
     // Initiate VEML6075.
     InitVEML6075();
     Delay_ms(150);
     
     // Display Address
     TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_GREEN, FO_HORIZONTAL);
     TFT_Write_Text("VEML6075_Address_is: ",30,50);
     IntToStr(get16(VEML6075_UV_ID), display);
     TFT_Write_Text(display, 265, 50);
     Delay_ms(3000);
     
     while(1)
     {
     // Get the data.
      for (i = 0; i < 5; i++)
      {
          UVData[i] = get16(VEML6075_UVA_DATA + i);
      }
      // Set up Screen 2.
      TFT_Fill_Screen(CL_BLACK);
      TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_YELLOW, FO_HORIZONTAL);
      TFT_Write_Text("UVA:", 0, 0);
      TFT_Write_Text("UVD:", 0, 20);
      TFT_Write_Text("UVB:", 0, 40);
      TFT_Write_Text("UVC1:", 0, 60);
      TFT_Write_Text("UVC2:", 0, 80);
      TFT_Write_Text("UVAi:", 0, 100); //microWatts/cm^2
      TFT_Write_Text("UVBi:", 0, 120); //microWatts/cm^2
      TFT_Write_Text("UVI:", 0, 140);
      // Show the datas we got.
      TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLUE, FO_HORIZONTAL);
      for (i = 0; i < 5; i++)                              //UVA, UVD, UVB, UVC1, UVC2.
      {
       IntToStr(UVData[i], display);
       TFT_Write_Text(display, 60, i * 20);
      }
      FloatToStr(getUVAIntensity(UVData[0]), display);     //UVA Intensity
      TFT_Write_Text(display, 60, 100);
      
      FloatToStr(getUVBIntensity(UVData[2]), display);     //UVB Intensity
      TFT_Write_Text(display, 60, 120);
      
      UVi = getUVIndex(UVData);   //Get the UV Index.
      FloatToStr(UVi, display);                                              //Display Index Value.
      TFT_Write_Text(display, 60, 140);

      TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
      // Translate index value.
      if (UVi <= 2)
      {
         TFT_Write_Text("UV is LOW", 90, 170);
      }
      else if (UVi > 2 && UVi <= 6)
      {
              TFT_Write_Text("UV is MODERATE", 60, 170);
      }
      else if (UVi > 6 && UVi <= 8)
      {
              TFT_Write_Text("UV is HIGH", 90, 170);
      }
      else if (UVi > 8 && UVi <= 10)
      {
              TFT_Write_Text("UV is VERY HIGH", 60, 170);
      }
      else if (UVI > 10)
      {
              TFT_Write_Text("UV is EXTREME", 85, 170);
      }
         sprintf(display, "UV Index is:  %2.1f",UVi); //Simplify Index and display.
         TFT_Write_Text(display, 85, 190);
      
      Delay_ms(1000);
     }
}

//Initiate VEML6063
void InitVEML6075()
     {
      I2C2_Start();
      I2C2_Write(VEML6075_WADD);
      I2C2_Write(VEML6075_UV_CONF);
      I2C2_Write(IT << 4);//LSB
      I2C2_Write(0x00);   //MSB
      I2C2_Stop();
     }

// Get 16bit data I2C.
unsigned int get16(unsigned int reg)
     {
        unsigned int rawData[2];
        I2C2_Start();
        I2C2_Write(VEML6075_WADD);
        I2C2_Write(reg); //Register
        I2C2_Restart();
        I2C2_Write(VEML6075_RADD);
        rawData[0] = I2C2_Read(0);//LSB
        rawData[1] = I2C2_Read(0);//MSB
        I2C2_Stop();
        return rawData[1] << 8 | rawData[0];  //MSB LSB
    }
    //microWatts/cm^2
    float getUVAIntensity(int _UVA)
    {
          float UVASensitivity = (0.93 / (IT + 1)); //UVA Sensitivity
          return _UVA / UVASensitivity;
    }
     //microWatts/cm^2
    float getUVBIntensity(int _UVB)
    {
          float UVBSensitivity = 2.10/(IT + 1); //UVB Sensitivity
          return  _UVB / UVBSensitivity;
    }

    //f getUVIndex(int Array)
    //Takes in UV variable array for UV calculation
    //Returns UV index (Float).
    float getUVIndex(unsigned int n[])
    {
          unsigned int _UVA  = n[0];
          unsigned int _UVD  = n[1];
          unsigned int _UVB  = n[2];
          unsigned int _UVC1 = n[3];
          unsigned int _UVC2 = n[4];
          float UVAcomp = (_UVA - _UVD) - A *(_UVC1 - _UVD) - B *(_UVC2 - _UVD);
          float UVBcomp = (_UVB - _UVD) - C *(_UVC1 - _UVD) - D *(_UVC2 - _UVD);
          return ((UVBcomp * UVBresponse) + (UVAcomp * UVAresponse) ) /2;;
    }